#include <stdexcept>
#include "circArrayIntList.h"
using namespace std;


CircArrayIntList::CircArrayIntList() {
    this->theArray = new int[10];
    this->size = 0;
    this->capacity = 10;
    this->startPosition = 0;
}

CircArrayIntList::CircArrayIntList(CircArrayIntList &toCopy) {
    this->size = toCopy.size;
    this->capacity = toCopy.capacity;
    this->theArray = new int[this->capacity];
    this->startPosition = toCopy.startPosition;
    // i just counts up to this->size
    // j is the index in the array;
    int i,j;
    j = this->startPosition;
    for (i=0; i<this->size && j < this->capacity; i++,j++) {
        this->theArray[j] = toCopy.theArray[j];
    }
    for (j=0; i<this->size; i++,j++) {
        this->theArray[j] = toCopy.theArray[j];
    }
}

CircArrayIntList::~CircArrayIntList() {
    delete[] this->theArray;
}

int CircArrayIntList::getSize() {
    return this->size;
}

bool CircArrayIntList::isEmpty() {
    return this->size == 0;
}

int CircArrayIntList::peekHead() {
    if (this->size == 0) {
        throw runtime_error("CircArrayIntList::peekHead, list is empty");
    } else {
        return this->theArray[this->startPosition];
    }
}

int CircArrayIntList::peekTail() {
    if (this->size == 0) {
        throw runtime_error("CircArrayIntList::peekTail, list is empty");
    } else {
        int endPosition = this->startPosition + this->size - 1;
        if (endPosition >= this->capacity) {
            endPosition = endPosition - this->capacity;
        }
        return this->theArray[endPosition];
    }
}

int& CircArrayIntList::at(int i) {
    if (i >= 0 && i < size) {
        int atPosition = this->startPosition + i;
        if (atPosition >= this->capacity) {
            atPosition = atPosition - this->capacity;
        }
        return this->theArray[atPosition];
    } else {
        throw runtime_error("CircArrayIntList::at, not a valid index.");
    }
}

int& CircArrayIntList::operator[](int i) {
    int atPosition = this->startPosition + i;
    if (atPosition >= this->capacity) {
        atPosition = atPosition - this->capacity;
    }
    return this->theArray[atPosition];
}

void CircArrayIntList::insertAtHead(int value) {
    if (this->size == this->capacity) {
        expandCapacity();
    }
    this->startPosition--;
    if (this->startPosition < 0) {
        this->startPosition = this->startPosition + this->capacity;
    }
    this->theArray[this->startPosition] = value;
    this->size++;
}

void CircArrayIntList::insertAtTail(int value) {
    if (this->size == this->capacity) {
        expandCapacity();
    }
    int endPosition = this->startPosition + this->size;
    if (endPosition >= this->capacity) {
        endPosition = endPosition - this->capacity;
    }
    this->theArray[endPosition] = value;
    this->size++;
}

int CircArrayIntList::removeHead() {
    if (this->size == 0) {
        throw runtime_error("CircArrayIntList::peekTail, list is empty");
    }
    int tmp = this->theArray[this->startPosition];
    this->startPosition++;
    if (this->startPosition >= this->capacity) {
        this->startPosition = this->startPosition - this->capacity;
    }
    this->size--;
    return tmp;
}

int CircArrayIntList::removeTail() {
    if (this->size == 0) {
        throw runtime_error("CircArrayIntList::peekTail, list is empty");
    } else {
        int endPosition = this->startPosition + this->size - 1;
        if (endPosition >= this->capacity) {
            endPosition = endPosition - this->capacity;
        }
        this->size--;
        return this->theArray[endPosition];
    }
}

void CircArrayIntList::expandCapacity() {
    int newCapacity = this->capacity * 2;
    int* newArray = new int[newCapacity];
    // i counts the number of elements copied
    // j is the index in the array
    int i,j;
    // determine how many iterations we can do before wrapping
    int beforeWrapping = this->capacity - this->startPosition;
    if (beforeWrapping > this->size) {
        // no wrapping necessary
        beforeWrapping = this->size;
    }

    j = this->startPosition;
    // the new array will have 0 as its new startPosition
    for (i=0; i<beforeWrapping; i++,j++) {
        newArray[i] = this->theArray[j];
    }
    for (j=0; i<this->size; i++,j++) {
        newArray[i] = this->theArray[j];
    }
    delete this->theArray;
    this->theArray = newArray;
    this->capacity = newCapacity;
    this->startPosition = 0;
}

CircArrayIntList& CircArrayIntList::operator=(const CircArrayIntList& toAssign) {
    if (this != &toAssign) {
        delete[] this->theArray;

        this->size = toAssign.size;
        this->capacity = toAssign.capacity;
        this->theArray = new int[this->capacity];
        this->startPosition = toAssign.startPosition;
        // i just counts up to this->size
        // j is the index in the array;
        int i,j;
        j = this->startPosition;
        for (i=0; i<this->size && j < this->capacity; i++,j++) {
            this->theArray[j] = toAssign.theArray[j];
        }
        for (j=0; i<this->size; i++,j++) {
            this->theArray[j] = toAssign.theArray[j];
        }
    }
    return *this;
}